#!/usr/bin/perl -I ../../external/perl5


use strict;
use Getopt::Std;
use File::Path;
use Cwd 'realpath';

my $HOME = "/afs/cs/u/olga/";
my $STAIR = "/afs/cs/group/stair/";

my $dir = "${HOME}new_lasik/experiments/";

my $script="${HOME}new_lasik/svl/scripts/fullyTrainObjectDetector.pl";

my $dataDir="${STAIR}data/visionDataset/";
my $objectsDir="${STAIR}data/objects/";
my $outputDir = "${HOME}scratch/experiments/";
my $modelDir = "${HOME}scratch/models/";

my @objects = qw(bottle can clock cup door_handle keyboard monitor mug ski_boot wastebasket);
my @widths =  qw(32 32 32 32 40 96 64 32 64 32);
my @heights = qw(64 64 32 40 40 32 64 32 64 43);

my $params = "-r ${dataDir}olga/image_seq_neg.xml -t ${dataDir}olga/image_seq_rest.xml -G ${dataDir}video_objects.xml -c 2 -b 500";

my @labels = qw(intensity noedge coloredge);

#my @labels = qw(PbothNwith PtrainNwith PgoogleNwith PbothNwithout PgoogleNwithout PtrainNwithout );

#my @labels = qw(final);
my $fs = "2";

my $FILE;

for (my $j = 0; $j < @objects; $j++) {
    my $object = @objects[$j];

    for (my $i = 0; $i < @labels; $i++) {
	my $label = @labels[$i];
	
	my $filename = "${dir}train.".${label}.".${object}.sh";
	
	open(FILE, ">$filename");
	print FILE "#!/bin/sh\n";

	#print STDERR "$filename\n";

	my $cmd = "$script $params -o ${outputDir}${label} -m ${modelDir}${label} -f $fs"; # "$script $params -i $i -o ${outputDir}$label";
	$cmd = "$cmd -w @widths[$j] -h @heights[$j]";

	if ($object eq "clock" || $object eq "door_handle" || $object eq "cup" || 
	    $object eq "wastebasket" || $object eq "ski_boot") {
	    $cmd = "$cmd -p ${objectsDir}${object}"; 
	}
	print FILE "$cmd $object >& ${outputDir}${label}/${object}.commands\n";

    }

    close(FILE);
}

